
import java.lang.reflect.Array;
import java.util.*;

public class Graphe {

    //graphe non orienté, indice des sommets peut etre quelconque

    //sommet i à voisins adj.get(i)
    //numéros de sommets pas forcément consécutifs,
    private HashMap<Integer,HashSet<Integer>> adj;
    private int m;

    public Graphe(int n){
        this(n,false,0);
    }
    public Graphe(int n, boolean random, double p) {
        //construit à graphe à n sommets 0..n-1
        adj = new HashMap<Integer, HashSet<Integer>>();
        for(int i=0;i<n;i++){
            adj.put(i,new HashSet<>());
        }
        m=0;

        if(random){
            Random r = new Random();
            for(int i=0;i<n;i++){
                for(int j=0;j<n;j++){
                    if(i!=j){
                        if(r.nextDouble()<p){
                            ajoutArete(i,j);
                        }
                    }

                }
            }
        }
    }


    public Graphe(Graphe g){
        m=g.m;
        adj = new HashMap<>();
        for(Map.Entry<Integer,HashSet<Integer>> e : g.adj.entrySet()){
            HashSet<Integer> voisins = e.getValue();
            HashSet<Integer> copie = new HashSet<>();
            for(int v : voisins)
                copie.add(v);
            adj.put(e.getKey(),copie);
        }

    }
    public int n() {
        return adj.size();
    }

    public Set<Integer> sommets(){
        return adj.keySet();
    }
    public int m(){return m;}


    public Arete getArete(){
        for(Map.Entry<Integer,HashSet<Integer>> e : adj.entrySet()){
            if(!e.getValue().isEmpty()){
                for(int v : e.getValue()){
                    return new Arete(e.getKey(),v);
                }
            }
        }
        return null;
    }
    public void supprimeArete(int i, int j) {
        //prérequis 0 <= i,j < nbSommets()

            adj.get(i).remove(j);
            adj.get(j).remove(i);
            m--;
    }


    public int chercheDegUn(){
        for(Map.Entry<Integer,HashSet<Integer>> e : adj.entrySet()){
            if(e.getValue().size()==1){
                return e.getKey();
            }
        }
        return -1;
    }
    public int chercheDegStrictK(int k){
        for(Map.Entry<Integer,HashSet<Integer>> e : adj.entrySet()){
            if(e.getValue().size()>k){
                return e.getKey();
            }
        }
        return -1;
    }
    public void supprimeSommet(int i){
        adj.remove(i);
        for(Map.Entry<Integer,HashSet<Integer>> e : adj.entrySet()){
            if(e.getValue().remove(i)){
                m--;
            }
        }
    }
    public boolean ajoutSommet(int i){
        //si i pas déjà présent, l'ajout et retourne vrai, sinon ne fait rien et retourne faux
        if(adj.containsKey(i))
            return false;
        else{
            adj.put(i,new HashSet<>());
            return true;
        }
    }

    public Set<Integer> getVoisins(int i){
        //prerequis i present
        return adj.get(i);
    }
    public void ajoutArete(int i, int j) {
        //prérequis
        //i et j existent
        //i != j

            if(!existeArete(i,j)) {
                adj.get(i).add(j);
                adj.get(j).add(i);

                m++;
            }
    }

    public void ajoutAretes(int i, Set<Integer> X) {
        //prérequis
        //i et tous sommets de X existent
        //i != j

        for(Integer j : X)
            ajoutArete(i,j);
    }

    public boolean existeArete(int i, int j) {
        return adj.containsKey(i) && adj.get(i).contains(j);
    }



    public ArrayList<Arete> matching(){
        //trouve un matching maximal gloutonnement
        if(m==0){
            return new ArrayList<>();
        }
        Arete a = getArete();
        supprimeSommet(a.getJ());
        supprimeSommet(a.getI());
        ArrayList<Arete> res = matching();
        res.add(a);
        return res;
    }
    public String toString() {

        return adj.toString();
    }



    public static void main(String[] args) {


    }

}